import UIKit
import CoreLocation

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {

    var window: UIWindow?

    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplicationLaunchOptionsKey: Any]?) -> Bool {

        let storyboard = UIStoryboard(name: "Main", bundle: nil)
        let navigationVC = window!.rootViewController as! UINavigationController

        // if the user hasn't decided yet about the Location Services access for this app, show the onboarding screen and ask for permission
        if CLLocationManager.authorizationStatus() != .notDetermined {
            let loginVC = storyboard.instantiateViewController(withIdentifier: "Login")
            navigationVC.viewControllers = [loginVC]
        }

        // attempt to restore the logged-in customer data upon app re-launch
        Customer.loadCurrent()

        // if we did restore an already logged-in customer, skip the login screen...
        if let customer = Customer.current {
            var vc: UIViewController!
            if customer.ticket != nil {
                // ...and go to straight to the "here's your tickets" screen, if they have any
                vc = storyboard.instantiateViewController(withIdentifier: "Tickets")
            } else {
                // ...or go to the "buy tickets" screen if they don't
                vc = storyboard.instantiateViewController(withIdentifier: "BuyTickets")
            }
            navigationVC.pushViewController(vc, animated: false)
        }

        // start scanning for beacons
        TicketCheckObserver.shared.start()

        return true
    }

    func applicationWillResignActive(_ application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(_ application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    }

    func applicationWillEnterForeground(_ application: UIApplication) {
        // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
    }

    func applicationDidBecomeActive(_ application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    }

    func applicationWillTerminate(_ application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    }
}
