import Foundation
import EstimoteProximitySDK

// here be Proximity code, if that's what you're looking for (-:

class TicketCheckObserver {

    static let shared = TicketCheckObserver()

    private(set) var currentMovieTheaterId: String?

    private let proximityObserver: EPXProximityObserver
    private let proximityZone: EPXProximityZone

    init() {
        let credentials = EPXCloudCredentials(appID: ESTIMOTE_APP_ID, appToken: ESTIMOTE_APP_TOKEN)

        proximityObserver = EPXProximityObserver(credentials: credentials, errorBlock: { error in
            NSLog("ProximityObserver error: \(error)")
        })


        // we want to detect when we're within roughly 3 meters of any beacons configured as "type: ticket-check"
        // naturally, make sure you've set up the "type: ticket-check" attachment on at least one of your beacons in Estimote Cloud
        //
        // in an actual deployment, you'd probably want to use a wider range, but for testing, let's go with something smaller; this is especially important if you're testing with multiple beacons (that is, simulating multiple theaters), to avoid overlap
        //
        // note that the trigger distance is just a rough estimate! the actual distance may vary from event to event

        proximityZone = EPXProximityZone(range: EPXProximityRange(desiredMeanTriggerDistance: 3.0)!, attachmentKey: "type", attachmentValue: "ticket-check")

        proximityZone.onEnterAction = { [unowned self] attachment in
            // we've entered the "ticket-check" zone, let's see which theater this is
            guard let movieTheaterId = attachment.payload["movieTheaterId"] as? String else {
                NSLog("no movieTheaterId set for beacon \(attachment.deviceIdentifier), make sure to set some in Estimote Cloud!")
                return
            }

            NSLog("TicketCheckObserver enter, movieTheaterId = \(movieTheaterId)")

            self.currentMovieTheaterId = movieTheaterId

            if let customer = Customer.current {
                // let's notify the Employee apps in the same theater about the customer
                postAction(.addOrUpdate, customer: customer, movieTheaterId: movieTheaterId)
            }
        }

        proximityZone.onExitAction = { [unowned self] attachment in
            // we've exited the "ticket-check" zone, let's see which theater was that
            guard let movieTheaterId = attachment.payload["movieTheaterId"] as? String else {
                NSLog("no movieTheaterId set for beacon \(attachment.deviceIdentifier), make sure to set some in Estimote Cloud!")
                return
            }

            NSLog("TicketCheckObserver exit, movieTheaterId = \(movieTheaterId)")

            self.currentMovieTheaterId = nil

            if let customer = Customer.current {
                // let's notify the Employee apps in that theater that the customer has left
                postAction(.remove, customer: customer, movieTheaterId: movieTheaterId)
            }
        }
    }

    func start() {
        proximityObserver.startObserving([proximityZone])
    }
}
