import UIKit
import EstimoteProximitySDK
import CoreLocation

// This VC is to detect which theater the Employee app is in. Once we find out (see the onEnterAction below), we transition to the Tickets VC.
//
// This only happens when running on a physical iOS device. If run in a simulator (which doesn't support Bluetooth = doesn't support beacons = doesn't support Proximity), we'll instead assume we're in SIMULATOR_MOVIE_THEATER_ID (see CHANGE_ME.swift), and start straight in the Tickets VC (see AppDelegate).

class LocationViewController: UIViewController {

    var proximityObserver: EPXProximityObserver!
    var proximityZone: EPXProximityZone!

    var movieTheaterId: String?

    override func viewDidLoad() {
        super.viewDidLoad()

        let locationManager = CLLocationManager()
        locationManager.requestWhenInUseAuthorization()

        let credentials = EPXCloudCredentials(appID: ESTIMOTE_APP_ID, appToken: ESTIMOTE_APP_TOKEN)

        proximityObserver = EPXProximityObserver(credentials: credentials, errorBlock: { error in
            NSLog("ProximityObserver error: \(error)")
        })

        proximityZone = EPXProximityZone(range: .far, attachmentKey: "type", attachmentValue: "ticket-check")
        proximityZone.onEnterAction = { [unowned self] attachment in
            if let movieTheaterId = attachment.payload["movieTheaterId"] as? String {
                NSLog("location detected: \(movieTheaterId)")

                self.proximityObserver.stopObservingZones()

                self.movieTheaterId = movieTheaterId
                self.performSegue(withIdentifier: "LocationFound", sender: nil)
            }
        }
    }

    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        proximityObserver.startObserving([proximityZone])
    }

    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        guard let movieTheaterId = movieTheaterId else { return }
        let ticketsVC = segue.destination as! TicketsViewController
        ticketsVC.movieTheaterId = movieTheaterId
    }
}
