import UIKit
import SocketIO

// This here is just a simple Table View VC, which subscribes to the backend via Socket.io. This way, it'll be notified whenever a customer enters or exits the ticket-check area, and can update the Table View.

class TicketsViewController: UITableViewController {

    var movieTheaterId: String!

    var socketManager: SocketManager!

    var customers = [Customer]()

    override func viewDidLoad() {
        super.viewDidLoad()

        socketManager = SocketManager(socketURL: URL(string: GLITCH_APP_URL)!,
                                      config: [.log(true),
                                               .connectParams(["movieTheaterId": movieTheaterId!])])

        let socket = socketManager.defaultSocket

        socket.on("addOrUpdate") { [unowned self] data, ack in
            guard let customer = decodeCustomer(socketIOData: data) else { return }

            if let index = self.customers.index(where: { $0.name == customer.name }) {
                // we already have this customer on our list, so simply update its data
                self.customers[index] = customer
                self.tableView.reloadRows(at: [IndexPath(row: index, section: 0)], with: .right)
            } else {
                // we don't have that customer on our list yet, add it
                self.customers.append(customer)
                self.tableView.insertRows(at: [IndexPath(row: self.customers.count - 1, section: 0)], with: .right)
            }
        }

        socket.on("remove") { [unowned self] data, ack in
            guard let customer = decodeCustomer(socketIOData: data) else { return }

            // if we have that customer on our list, remove it
            if let index = self.customers.index(where: { $0.name == customer.name }) {
                self.customers.remove(at: index)
                self.tableView.deleteRows(at: [IndexPath(row: index, section: 0)], with: .right)
            }
        }

        socket.connect()
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return customers.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "TicketCell", for: indexPath) as! TicketCell

        let customer = customers[indexPath.row]
        cell.update(with: customer)

        return cell
    }
}

private func decodeCustomer(socketIOData: [Any]) -> Customer? {
    guard let jsonString = socketIOData[0] as? String,
        let jsonData = jsonString.data(using: .utf8)
        else { return nil }

    let decoder = JSONDecoder()
    let customer = try? decoder.decode(Customer.self, from: jsonData)
    return customer
}
