# Ticket Check example

Imagine you bought a movie ticket via a mobile app. You enter the movie theater's lobby, grab pop corn in one hand, soda in the other, and move towards the ticket check. 

> attendant: Hi! Can I have your name?
> you: It's Peter.
> attendant: <glances at their iPad> Thanks Peter, enjoy "Ready Player One"!

…and simply let's you through.

Mind reading? Not quite—the smartphone in your pocket detects that you're in the ticket-check area, and simply sends your ticket to the employee's iPad. No need for a weird dance trying to show your ticket while your hands are full of snacks!

## How to run it 🚀

1. Run `pod install` in this example's directory.

    If you're not familiar with CocoaPods, it's a popular iOS-development tool for managing external libraries/dependencies in Xcode projects—see their [Getting Started](https://guides.cocoapods.org/using/getting-started.html) guide for more.

2. In [Estimote Cloud](https://cloud.estimote.com), in the Apps section, add a new "your own app" to generate an App ID and Token. Use them in the CHANGE_ME files (note that there are two of these: one in the Customer project, one in the Employee project).

3. Still in Estimote Cloud, in the Beacons section, pick one beacon and configure it for use with this example:

    - Go to its "Settings", and then "Edit Settings"
    - Go to "Beacon Attachment"
    - Add two attachments: `type = ticket-check`, and `movieTheaterId` = `my-test-theater`
        - Note: since this example uses a shared backend by default, it's recommended to append a few random characters at the end for uniqueness, for example `my-test-theater-hgurnvid`. Otherwise, you'll likely "share" a "theater" with other users, that is, you'll see their names and tickets, and they will see yours.
        - Alternatively, see the comments in the CHANGE_ME files to learn how to use your own copy of the backend.

4. Build and deploy the Employee app to an iOS device, and put it together with your beacon some place, preferably away from your computer. You might want to adjust the screen settings so that it doesn't go to sleep, allowing you to observe what happens as you're gonna approach with the Customer app.

5. Build and deploy the Customer app to another iOS device. "Log in" with your name, "buy a ticket", then leave the app, lock the phone, and approach the beacon.

6. You should see your name and ticket pop up in the Employee app.

**Don't have two iOS devices?** In the CHANGE_ME file in the Employee project, set the SIMULATOR_MOVIE_THEATER_ID to the same `movieTheaterId` as your beacon. Run the Employee app in a simulator—the app will simply "assume" it's in range of that beacon. (The simulator doesn't support Bluetooth, so it can't *actually* detect beacons.)

## What's happening? How does this work? 🛠

When the Employee app starts, it detects the beacon and checks its `movieTheaterId` attachment. It then subscribes to the web backend to get notified about any customers in the ticket-check area in that theater.

When the Customer app enters the range of the beacon, it also checks the `movieTheaterId` attachment. It then sends a "hey, I'm here!" message to the backend, and the backend forwards it to the respective Employee apps. The message also contains the customer's name, and their ticket. The Employee app receives the message, and adds the customer's info to the list.

When the Customer app exits the range of the beacon, the process almost repeats itself, except this time it's a "hey, I'm leaving" message, and the Employee app removes the customer's info from the list.

For more details, we encourage you to study the code—we've tried our best to sprinke it with relevant comments. If something's still unclear, feel free to ask on [Estimote Forums](https://forums.estimote.com).

## What backend does this use? 🌍

It's a simple Node.js app built and hosted on glitch.com. You can see its source code here: 

https://glitch.com/edit/#!/movie-tickets-server

The easiest way to edit/tweak it, or to obtain your own instance (by default, everyone shares the same one), is to "Remix This" on Glitch. It's like forking a repo on GitHub: you'll get your own copy that you can edit and use, and Glitch will automatically run it for you. Make sure to update the URL in the CHANGE_ME files.

## Beyond just movie tickets 🎬

The idea behind this example can be generalized into many other use cases. Some examples:

- In a coffee shop, the cashier could automatically see your pre-paid rewards card, and charge your coffee to it. No barcodes to scan!
- In a restaurant, the host could automatically see your reservation, and lead you straight to your table.
- In a health clinic, the receptionist could automatically see your appointment, and point you to the doctor's office.

Any other ideas? Share with us on [Estimote Forums](https://forums.estimote.com).

## I can do this without beacons! 📡

Absolutely! Instead of proximity to beacons, you could use a regular, `CLCircularRegion` geo-fence around the theater. Or in the extreme, maybe there's no location services involved at all—the Employee simply has a full list of Customers with tickets for the upcoming screening.

This solution however is generally better the shorter and more precise the Employee's list is. With beacons, this can easily be only Customers within the ticket-check area. With `CLCircularRegion`, that'd be everyone around the theater area.

It's also more responsive: generally only about a few seconds to detect proximity to beacons, vs "3 to 5 minutes on average" (to quote the Core Location docs) for `CLCircularRegion`.  

Finally, using beacon-based location in the background has negligible impact on the smartphone's battery. With `CLCircularRegion`, being energy-efficient means using only WiFi and cellular signals, which further reduces accuracy and might be problematic if WiFi/cellular signal density is low—for example, outside urban areas.

### What about `CLBeaconRegion` AKA iBeacon? 🍏

Yup, that's definitely better than a `CLCircularRegion`. This example however uses Estimote's own Proximity technology. Here's how it compares to Core Location's `CLBeaconRegion` AKA iBeacon:

- **reliability**
    - Estimote Proximity: advanced beacon-signal stabilization, and also utilizes motion data, for more accurate and responsive (~ a few seconds) enter/exits
    - iBeacon: basic beacon-signal stabilization, 30-second exit delay

- **defining zones/regions**
    - Estimote Proximity: cloud-based attachments (here: "type = ticket-check")
    - iBeacon: UUID/major/minor physically configured on the beacon, and hard-coded in the app

- **zone/region's range**
    - Estimote Proximity: defined programatically in code (the "range" parameter in EPXProximityZone's initializer) 
    - iBeacon: depends on the beacon's physical broadcasting range

- Estimote Proximity: **cloud-based attachments** for extra data associated to a beacon (here: `movieTheaterId`)
    - no equivalent in Core Location (build your own cloud/backend, or hard-code the data in the app, or hard-code as iBeacon major/minor—short integers only)

- Estimote Proximity: optional **secure advertising** prevents unauthorized apps from also using your beacons for enter/exit events (e.g., any app with location permissions could track the user's visits in the theater), and also from spoofing the beacon (making the app believe you're in the theater, when you're not—somebody could potentially steal a ticket that way)
    - no equivalent in Core Location

- Estimote Proximity: **analytics** built-in, you can see them in Estimote Cloud (e.g., avg time people spend in the ticket-check)
    - no equivalent in Core Location

- Estimote Proximity: built-in **beacon health monitoring** and reporting, Estimote Cloud will notify you if a beacon is low on battery or went missing
    - no equivalent in Core Location
